'  ===========================================================================
'
'  BR404
'  the VSTi synthesizer for Windows systems
'
'  Finally available for your home computer:
'  The brandnew *farbrausch* software synthesizer VSTi  programmed by Benjamin BeRo Rosseaux.
'  Features
'
'      * many different oscillator waveforms and filters
'      * ADSR and freeform envelopes
'      * extensive modulation matrix
'      * sample exact modulations
'        (the main reason why BR404 may use slightly more CPU time than other comparable synths)
'      * sample support
'      * speech synthesizer
'      * PADsynth algorithm
'      * custom LFO waveforms per sample
'      * 10-band equalizer, Winamp compatible parameter presets
'      * multiple instances
'      * export song as win32 executable
'      * own fileformat (bmf2wav converter included)
'      * replay plugin for Winamp (and compatibles)
'      * SDK for Delphi 5/7, FreePascal 2.x and MSVC / MSVC COFF library file format compatible products included
'      *  and a whole lotta more for you to discover!
'
'      http://www.farbrausch.de/br404/
'
'  ===========================================================================
'
'  ===========================================================================
'   Freebasic compatible library and wrapper by rbz - 01 Feb 2009
'  ===========================================================================

#ifndef LIBBR404_H_
#define LIBBR404_H_

#include "windows.bi"
#include "win/mmsystem.bi"

#if __FB_VERSION__ < "0.17" 
    '$inclib: "winmm"
    '$inclib: "br404"
#else
    #inclib "winmm"
    #inclib "br404" 
#endif

'' ===========================================================================
''                          farbrausch br404 API                                
'' ===========================================================================

#define slNONE 0
#define slFORWARD 1
#define slPINGPONG 2
#define slBACKWARD 3

type SynthEvent
 Time as longint
 ID   as longint
 Command as byte
 Data(4) as byte
 Previous as SynthEvent ptr
 Next as SynthEvent ptr
 SampleIndex as longint
end type

Declare Function SynthCreate stdcall Alias "SynthCreate"(byval SampleRate as integer,byval BufferSize as integer,byval ThreadPlay as integer,byval SoundOutput as integer) as any ptr
Declare Sub SynthDestroy stdcall Alias "SynthDestroy"(byval Track as any ptr)

Declare Function SynthGetTrackSize stdcall Alias "SynthGetTrackSize"() as integer

Declare Function SynthInit stdcall Alias "SynthInit"(byval Track as any ptr,byval SampleRate as integer,byval BufferSize as integer,byval ThreadPlay as integer,byval SoundOutput as integer) as any ptr
Declare Function SynthReinit stdcall Alias "SynthReinit"(byval Track as any ptr,byval SampleRate as integer,byval BufferSize as integer) as any ptr
Declare Sub SynthDone stdcall Alias "SynthDone"(byval Track as any ptr)

Declare Sub SynthLink stdcall Alias "SynthLink"(byval Track as any ptr, byval ToTrack as any ptr)
Declare Sub SynthUnlink stdcall Alias "SynthUnlink"(byval Track as any ptr)

Declare Sub SynthSetAGC stdcall Alias "SynthSetAGC"(byval Track as any ptr, byval AGC as integer)
Declare Sub SynthSetClipping stdcall Alias "SynthSetClipping"(byval Track as any ptr, byval Clipping as integer)
Declare Sub SynthSetLooping stdcall Alias "SynthSetLooping"(byval Track as any ptr, byval Looping as integer)

Declare Function SynthGetTrackName stdcall Alias "SynthGetTrackName"(byval Track as any ptr) as byte ptr
Declare Function SynthGetTrackAuthor stdcall Alias "SynthGetTrackAuthor"(byval Track as any ptr) as byte ptr
Declare Function SynthGetTrackComments stdcall Alias "SynthGetTrackComments"(byval Track as any ptr) as byte ptr
Declare Function SynthGetBuffer stdcall Alias "SynthGetBuffer"(byval Track as any ptr) as any ptr
Declare Function SynthGetRootEvent stdcall Alias "SynthGetRootEvent"(byval Track as any ptr) as SynthEvent ptr

Type Hook as function stdcall (byval as any ptr) as any ptr
Declare Sub SynthSetBufferStartHook stdcall Alias "SynthSetBufferStartHook"(byval Track as any ptr, as Hook)
Declare Sub SynthSetBufferEndHook stdcall Alias "SynthSetBufferEndHook"(byval Track as any ptr, as Hook)
Declare Sub SynthSetEventHook stdcall Alias "SynthSetEventHook"(byval Track as any ptr, as Hook)

Declare Function SynthCreateTrackSample stdcall Alias "SynthCreateTrackSample"(byval Track as any ptr,byval LoopMode as integer,byval LoopStart as integer,byval LoopEnd as integer,byval SustainLoopMode as integer,byval SustainLoopStart as integer,byval SustainLoopEnd  as integer) as any ptr

Declare Sub SynthDestroyTrackSample stdcall Alias "SynthDestroyTrackSample"(byval TrackSample as any ptr)
Declare Function SynthCreateTrackSampleInstance stdcall Alias "SynthCreateTrackSampleInstance"(byval TrackSample as any ptr) as any ptr
Declare Sub SynthDestroyTrackSampleInstance stdcall Alias "SynthDestroyTrackSampleInstance"(byval TrackSampleInstance as any ptr)

Declare Sub SynthStartTrackSampleInstance stdcall Alias "SynthStartTrackSampleInstance"(byval TrackSampleInstance as any ptr)
Declare Sub SynthStopTrackSampleInstance stdcall Alias "SynthStopTrackSampleInstance"(byval TrackSampleInstance as any ptr)

Declare Sub SynthKeyOffTrackSampleInstance stdcall Alias "SynthKeyOffTrackSampleInstance"(byval TrackSampleInstance as any ptr)
Declare Sub SynthUpdateTrackSampleInstance stdcall Alias "SynthUpdateTrackSampleInstance"(byval TrackSampleInstance as any ptr)

Declare Sub SynthSetTrackSampleInstanceVolume stdcall Alias "SynthSetTrackSampleInstanceVolume"(byval TrackSampleInstance as any ptr, byval Volume as double)
Declare Sub SynthSetTrackSampleInstancePanning stdcall Alias "SynthSetTrackSampleInstancePanning"(byval TrackSampleInstance as any ptr, byval Panning as double)
Declare Sub SynthSetTrackSampleInstancePanningLeftRight stdcall Alias "SynthSetTrackSampleInstancePanningLeftRight"(byval TrackSampleInstance as any ptr, byval PanningLeft as double, byval PanningRight as double)
Declare Sub SynthSetTrackSampleInstancePitch stdcall Alias "SynthSetTrackSampleInstancePitch"(byval TrackSampleInstance as any ptr, byval Pitch as double)

Declare Sub SynthResetVoices stdcall Alias "SynthResetVoices"(byval Track as any ptr)
Declare Sub SynthResetChannels stdcall Alias "SynthResetChannels"(byval Track as any ptr)
Declare Sub SynthReset stdcall Alias "SynthReset"(byval Track as any ptr)

Declare Sub SynthAddEvent stdcall Alias "SynthAddEvent"(byval Track as any ptr, byval NewEvent as SynthEvent ptr) 

Declare Function SynthReadBankInformations stdcall Alias "SynthReadBankInformations"(byval Track as any ptr, byval _Data as byte ptr, byval DataSize as uinteger) as integer
Declare Function SynthReadBank stdcall Alias "SynthReadBank"(byval Track as any ptr, byval _Data as any ptr, byval DataSize as uinteger) as integer
Declare Function SynthReadSMF stdcall Alias "SynthReadSMF"(byval Track as any ptr, byval _Data as any ptr, byval DataSize as uinteger) as integer
Declare Function SynthReadBMFInformations stdcall Alias "SynthReadBMFInformations"(byval Track as any ptr, byval _Data as any ptr, byval DataSize as uinteger) as integer
Declare Function SynthReadBMFSampleRate stdcall Alias "SynthReadBMFSampleRate"(byval _Data as any ptr, byval DataSize as uinteger) as uinteger
Declare Function SynthReadBMF stdcall Alias "SynthReadBMF"(byval Track as any ptr, byval _Data as any ptr, byval DataSize as uinteger) as integer

Declare Sub SynthFreeTrack stdcall Alias "SynthFreeTrack"(byval Track as any ptr)
Declare Sub SynthClearTrack stdcall Alias "SynthClearTrack"(byval Track as any ptr)

Declare Sub SynthChangeTempo stdcall Alias "SynthChangeTempo"(byval Track as any ptr)
Declare Sub SynthChangeTempoEx stdcall Alias "SynthChangeTempoEx"(byval Track as any ptr, byval NewTicksPerQuarterNote as integer, byval NewTempo as integer)

Declare Sub SynthResetEventQuery stdcall Alias "SynthResetEventQuery"(byval Track as any ptr)

Declare Sub SynthFillBuffer stdcall Alias "SynthFillBuffer"(byval Track as any ptr, byval Samples as integer)
Declare Sub SynthFillOutputBuffer stdcall Alias "SynthFillOutputBuffer"(byval Track as any ptr, byval OutputData as any ptr)

Declare Function SynthSeekToSample stdcall Alias "SynthSeekToSample"(byval Track as any ptr, byval DestSample as longint) as longint
Declare Function SynthSeekToMS stdcall Alias "SynthSeekToMS"(byval Track as any ptr, byval DestMS as longint) as longint

Declare Function SynthTimePosition stdcall Alias "SynthTimePosition"(byval Track as any ptr) as longint
Declare Function SynthSamplePosition stdcall Alias "SynthSamplePosition"(byval Track as any ptr) as longint

Declare Function SynthGetTimePosition stdcall Alias "SynthGetTimePosition"(byval Track as any ptr) as longint
Declare Function SynthGetSamplePosition stdcall Alias "SynthGetSamplePosition"(byval Track as any ptr) as longint

Declare Sub SynthPoll stdcall Alias "SynthPoll"(byval Track as any ptr)

Declare Sub SynthCreateThread stdcall Alias "SynthCreateThread"(byval Track as any ptr)
Declare Sub SynthExitThread stdcall Alias "SynthExitThread"(byval Track as any ptr)

Declare Sub SynthPlay stdcall Alias "SynthPlay"(byval Track as any ptr)
Declare Sub SynthStop stdcall Alias "SynthStop"(byval Track as any ptr)

Declare Sub SynthEnter stdcall Alias "SynthEnter"(byval Track as any ptr)
Declare Sub SynthLeave stdcall Alias "SynthLeave"(byval Track as any ptr)


